%
%
%  Track Plot Length Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Tracks Functions">
%        <Item name="Plot Length of selected Track" icon="Matlab" tooltip="Plot the length of the track steps.">
%          <Command>MatlabXT::XTTrackPlotLength(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Plot Length of selected Track" icon="Matlab" tooltip="Plot the length of the track steps.">
%            <Command>MatlabXT::XTTrackPlotLength(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Plot Length of selected Track" icon="Matlab" tooltip="Plot the length of the track steps.">
%            <Command>MatlabXT::XTTrackPlotLength(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Plot the length of the track steps. 
%   
%

function XTTrackPlotLength(aImarisApplicationID,aThreshold)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some tracks
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some tracks in the surpass scene!');
  return
end

% get the track
vObjectIndices = GetFirstSelectedTrack(vImarisApplication);
if isempty(vObjectIndices)  
  msgbox('Please select a track!');
  return
end

if nargin<2
  vAnswer = inputdlg(['Step length below the threshold is considered ', ...
    'as "pause" indicator. Please enter the threshold value:'], ...
    'Plot track length', ...
    1, {'0.4'});
  if isempty(vAnswer), return, end
  vThreshold = sscanf(char(vAnswer), '%f');
else
  vThreshold = aThreshold;
end

[vCoords, vTimes] = GetObjectsCoordsTimes( ...
  vImarisApplication, vObjectIndices);

vNumberOfObjects = numel(vTimes);

% order eventual unordered data (e.g. vTimes == [1,5,2])
vTimeMin = min(vTimes);
vTimeMax = max(vTimes);

vObjectAtTime = zeros(1, vTimeMax - vTimeMin + 1);
vObjectAtTime(vTimes - vTimeMin + 1) = 1:vNumberOfObjects; % [1,3,0,0,2]
vObjectsIndex = vObjectAtTime(vObjectAtTime ~= 0); % [1,3,2]

% computes the track edges
vPrev = vObjectsIndex(1:vNumberOfObjects-1);
vNext = vObjectsIndex(2:vNumberOfObjects);
vStepsLength = vCoords(vNext, :) - vCoords(vPrev, :);
vTimesLength = double(max(vTimes(vNext) - vTimes(vPrev), 1));

vStepsSize = sqrt(sum( (vStepsLength).^2, 2 )) ./ vTimesLength;

vPause = sum(vStepsSize <= vThreshold);

vTimes = vTimes(1:vNumberOfObjects-1) - vTimeMin + 1;

subplot(2,2,1)
plot(vTimes, vStepsSize, 'b-')
hold on
plot(vTimes, zeros(1, vNumberOfObjects-1) + vThreshold, 'r-')
hold off
vName = char(vImarisApplication.GetSurpassSelection.GetName);
title(sprintf('Steps size of %s (total length = %.2f)', vName, sum(vStepsSize)));
xlabel(sprintf('Number of pause steps = %i (%.2f%%)', vPause, ...
  vPause/(vNumberOfObjects-1)*100));
ylabel('Step size [um]');

% finally plot the results
vStrings = {'X', 'Y', 'Z'};
for vPlot = 1:3
    subplot(2,2, vPlot+1)
    plot(vTimes, vStepsLength(:, vPlot), 'b-');
    title([vStrings{vPlot}, '-Step size of ', vName]);
    xlabel('Time');
    ylabel('Step size [um]');
end


%---------------------------------------------------------%

function aObjectIndices = GetFirstSelectedTrack(aImarisApplication)

% aTracks is the list of indices (starting from 0) of the objects
%   in the track, *not* a set of edges
aObjectIndices = [];

% get the selected object (spots or surfaces)
vFactory = aImarisApplication.GetFactory;
vObject = vFactory.ToSpots(aImarisApplication.GetSurpassSelection);
if ~isempty(vObject)
  vSize = numel(vObject.GetIndicesT);
else
  vObject = vFactory.ToSurfaces(aImarisApplication.GetSurpassSelection);
  if ~isempty(vObject)
    vSize = vObject.GetNumberOfSurfaces;
  else
    return
  end
end

% get the sub-selection and the traks
vSelection = vObject.GetSelectedIndices;
vEdges = vObject.GetTrackEdges';
if isempty(vSelection) || isempty(vEdges)
  return
end

% get the first selected spot which belongs to a of the track
vCurrent = [];
vIndex = 0;
while ~any(vCurrent) && vIndex < numel(vSelection)
  vIndex = vIndex + 1;
  vCurrent = any(vEdges == vSelection(vIndex));
end
if ~any(vCurrent(:))
  % no spots selected belongs to a track
  return
end

% get the spots of the track of the first selected object
vTrackBool = false(vSize, 1);
vTrackBool(vSelection(vIndex) + 1) = true;
while any(vCurrent)
  vObjects = vEdges([vCurrent; vCurrent]);
  vTrackBool(vObjects + 1) = true;
  vEdges([vCurrent; vCurrent]) = vSize;
  vCurrent = false(size(vCurrent));
  for vIndex = 1:numel(vObjects)
    vCurrent = vCurrent | any(vEdges == vObjects(vIndex));
  end
end
vIndices = 0:vSize-1;
aObjectIndices = vIndices(vTrackBool);


%---------------------------------------------------------%

function [aCoords, aTimes] = GetObjectsCoordsTimes( ...
  aImarisApplication, aSelectionIndices)

aCoords = [];
aTimes = [];

% get the selected object (spots or surfaces)
vFactory = aImarisApplication.GetFactory;
vSpots = vFactory.ToSpots(aImarisApplication.GetSurpassSelection);
vSurfaces = vFactory.ToSurfaces(aImarisApplication.GetSurpassSelection);

if ~isempty(vSpots)
  aCoords = vSpots.GetPositionsXYZ;
  aTimes = vSpots.GetIndicesT;
  vSize = numel(aTimes);
  % remove invalid selection indices
  aSelectionIndices = aSelectionIndices(aSelectionIndices < vSize) + 1;
  % perform selection
  aCoords = aCoords(aSelectionIndices, :);
  aTimes = aTimes(aSelectionIndices);
elseif ~isempty(vSurfaces)
  vSize = vSurfaces.GetNumberOfSurfaces;
  % remove invalid selection indices
  aSelectionIndices = aSelectionIndices(aSelectionIndices < vSize);
  vSize = numel(aSelectionIndices);
  aCoords = zeros(vSize, 3);
  aTimes = zeros(vSize, 1);
  % read selection
  for vIndex = 1:vSize
    vSurface = aSelectionIndices(vIndex);
    aCoords(vIndex, :) = vSurfaces.GetCenterOfMass(vSurface);
    aTimes(vIndex) = vSurfaces.GetTimeIndex(vSurface);
  end
end
